@@ -11,10 +11,14 @@ from typing import Any, Dict, List
 import click
 
 import solitude.core
-from solitude.core import log_setup, get_connected_ssh_client_from_config
+from solitude.core import (
+    log_setup,
+    get_connected_ssh_client_from_config,
+)
 from solitude.config import Config
 from solitude.cache import json_write_atomic
 from solitude import TOOL_NAME, __version__
+from solitude.log import ActionTimeLogger
 from solitude.slurmjob import SlurmJob
 from solitude.utils.ssh import SSHClient
 
@@ -251,39 +255,42 @@ def run(
     ignore_errors: bool,
 ):
     if are_jobids_specified(jobids=jobids) and is_config_valid():
-        solitude.core.run(
-            cmd_files=cmd_files,
-            jobids=jobids,
-            workers=workers,
-            user=user,
-            priority=priority,
-            reservation=reservation,
-            duration=duration,
-            ignore_errors=ignore_errors,
-            cfg=DEFAULT_CONFIG,
-        )
+        with ActionTimeLogger():
+            solitude.core.run(
+                cmd_files=cmd_files,
+                jobids=jobids,
+                workers=workers,
+                user=user,
+                priority=priority,
+                reservation=reservation,
+                duration=duration,
+                ignore_errors=ignore_errors,
+                cfg=DEFAULT_CONFIG,
+            )
 
 
 @click.command(cls=SharedCommand, help="Extend running jobs")
 def extend(cmd_files: List[Path], jobids: List[int], workers: int):
     if are_jobids_specified(jobids=jobids) and is_config_valid():
-        solitude.core.extend(
-            cmd_files=cmd_files,
-            jobids=jobids,
-            workers=workers,
-            cfg=DEFAULT_CONFIG,
-        )
+        with ActionTimeLogger():
+            solitude.core.extend(
+                cmd_files=cmd_files,
+                jobids=jobids,
+                workers=workers,
+                cfg=DEFAULT_CONFIG,
+            )
 
 
 @click.command(cls=SharedCommand, help="Stop running jobs")
 def stop(cmd_files: List[Path], jobids: List[int], workers: int):
     if are_jobids_specified(jobids=jobids) and is_config_valid():
-        solitude.core.stop(
-            cmd_files=cmd_files,
-            jobids=jobids,
-            workers=workers,
-            cfg=DEFAULT_CONFIG,
-        )
+        with ActionTimeLogger():
+            solitude.core.stop(
+                cmd_files=cmd_files,
+                jobids=jobids,
+                workers=workers,
+                cfg=DEFAULT_CONFIG,
+            )
 
 
 @click.command(cls=SharedCommand, help="Link jobs to existing slurm job")
@@ -299,40 +306,42 @@ def link(
     cmd_files: List[Path], jobids: List[int], workers: int, slurm_jobid: int
 ):
     if are_jobids_specified(jobids=jobids) and is_config_valid():
-        cfg = DEFAULT_CONFIG
-        ssh_client = get_connected_ssh_client_from_config(
-            cfg=cfg, verbose=False
-        )
-        if SlurmJob.check_if_job_exists(
-            ssh_client=ssh_client, jobid=slurm_jobid
-        ):
-            for jobid in jobids:
-                success = solitude.core.link(
-                    cmd_files=cmd_files,
-                    jobid=jobid,
-                    slurm_jobid=slurm_jobid,
-                    cfg=cfg,
-                )
-                click.echo(
-                    f"Linked job id {jobid} to slurm job: {slurm_jobid} {'succeeded' if success else 'failed'}"
-                )
-        else:
-            click.echo(f"Slurm job id: {slurm_jobid} does not exists...")
+        with ActionTimeLogger():
+            ssh_client = get_connected_ssh_client_from_config(
+                cfg=DEFAULT_CONFIG, verbose=False
+            )
+            if SlurmJob.check_if_job_exists(
+                ssh_client=ssh_client, jobid=slurm_jobid
+            ):
+                for jobid in jobids:
+                    success = solitude.core.link(
+                        cmd_files=cmd_files,
+                        jobid=jobid,
+                        slurm_jobid=slurm_jobid,
+                        cfg=DEFAULT_CONFIG,
+                    )
+                    click.echo(
+                        f"Linked job id {jobid} to slurm job: {slurm_jobid} "
+                        f"{'succeeded' if success else 'failed'}"
+                    )
+            else:
+                click.echo(f"Slurm job id: {slurm_jobid} does not exists...")
 
 
 @click.command(cls=SharedCommand, help="Unlink jobs from slurm job")
 def unlink(cmd_files: List[Path], jobids: List[int], workers: int):
     if are_jobids_specified(jobids=jobids) and is_config_valid():
-        for jobid in jobids:
-            success = solitude.core.unlink(
-                cmd_files=cmd_files, jobid=jobid, cfg=DEFAULT_CONFIG
-            )
-            if success:
-                click.echo(f"Unlinked job id {jobid} from slurm jobs.")
-            else:
-                click.echo(
-                    f"Job id {jobid} has no slurm job associated with it."
+        with ActionTimeLogger():
+            for jobid in jobids:
+                success = solitude.core.unlink(
+                    cmd_files=cmd_files, jobid=jobid, cfg=DEFAULT_CONFIG
                 )
+                if success:
+                    click.echo(f"Unlinked job id {jobid} from slurm jobs.")
+                else:
+                    click.echo(
+                        f"Job id {jobid} has no slurm job associated with it."
+                    )
 
 
 @click.command(cls=SharedCommand, help="Get log file for slurm job")
@@ -351,20 +360,21 @@ def getlog(
     active_polling: bool,
 ):
     if are_jobids_specified(jobids=jobids) and is_config_valid():
-        for jobid in jobids:
-            log_output = solitude.core.get_log(
-                cmd_files=cmd_files,
-                jobid=jobid,
-                cfg=DEFAULT_CONFIG,
-                active_polling=active_polling,
-            )
-            if log_output:
-                if not active_polling:
-                    click.echo(log_output)
-            else:
-                click.echo(
-                    f"Job id {jobid} has no log file associated with it."
+        with ActionTimeLogger():
+            for jobid in jobids:
+                log_output = solitude.core.get_log(
+                    cmd_files=cmd_files,
+                    jobid=jobid,
+                    cfg=DEFAULT_CONFIG,
+                    active_polling=active_polling,
                 )
+                if log_output:
+                    if not active_polling:
+                        click.echo(log_output)
+                else:
+                    click.echo(
+                        f"Job id {jobid} has no log file associated with it."
+                    )
 
 
 @click.command(name="list", cls=SharedCommand, help="List jobs")
@@ -384,22 +394,33 @@ def getlog(
     default=True,
     help="Hide progress bar while loading the list. Default=No",
 )
+@click.option(
+    "-d",
+    "--show-duplicates",
+    "show_duplicates",
+    flag_value=True,
+    default=False,
+    help="Display all lines mapping to the same command. Default=No",
+)
 def list_jobs(
     cmd_files: List[Path],
     jobids: List[int],
     workers: int,
     selected_only: bool,
     show_progress_bar: bool,
+    show_duplicates: bool,
 ):
     if is_config_valid():
-        solitude.core.list_jobs(
-            cmd_files=cmd_files,
-            jobids=jobids,
-            workers=workers,
-            selected_only=selected_only,
-            show_progress_bar=show_progress_bar,
-            cfg=DEFAULT_CONFIG,
-        )
+        with ActionTimeLogger():
+            solitude.core.list_jobs(
+                cmd_files=cmd_files,
+                jobids=jobids,
+                workers=workers,
+                selected_only=selected_only,
+                show_progress_bar=show_progress_bar,
+                show_duplicates=show_duplicates,
+                cfg=DEFAULT_CONFIG,
+            )
 
 
 @click.group(cls=BaseGroup, help="Job operations")
