@@ -1,7 +1,7 @@
 import logging
 import json
 from enum import Enum, unique
-from typing import Callable, Dict, List, NamedTuple, Optional, Tuple
+from typing import Any, Callable, Dict, List, NamedTuple, Optional, Tuple
 from pathlib import Path
 
 import appdirs
@@ -51,7 +51,7 @@ class Config(object):
     class Defaults(NamedTuple):
         username: str = "UNKNOWN"
         priority: str = "low"
-        workers: int = 8
+        workers: int = 10
         cmdfiles: Tuple = ()
 
     class SSHConfig(NamedTuple):
@@ -100,11 +100,27 @@ class Config(object):
         if not passed:
             self.logger.warning(
                 f"`{group}` invalid value for `{prop}` was found "
-                f"in config.\n{msg}\nResorting to default value."
+                f"in config.\n{msg}"
             )
-            del settings[group][prop]
         return passed
 
+    def _get_property_or_default(
+        self,
+        settings: Dict,
+        group: str,
+        prop: str,
+        default: Any,
+        test: Optional[Callable] = None,
+        msg: str = "must be a string value.",
+    ) -> Any:
+        if self._validate_property(
+            settings=settings, group=group, prop=prop, test=test, msg=msg
+        ):
+            return settings[group][prop]
+        else:
+            self.logger.warning(f"Resorting to default value: `{default}`.")
+            return default
+
     def is_config_present(self) -> bool:
         return self.config_path.is_file()
 
@@ -122,38 +138,46 @@ class Config(object):
                 if "cmdfiles" in defaults:
                     defaults["cmdfiles"] = tuple(defaults["cmdfiles"])
                 # validate defaults
-                self._validate_property(
-                    settings=cfg, group="defaults", prop="username"
+                self.defaults = Config.Defaults(
+                    username=self._get_property_or_default(
+                        settings=cfg,
+                        group="defaults",
+                        prop="username",
+                        default=Config.Defaults.username,
+                    ),
+                    priority=self._get_property_or_default(
+                        settings=cfg,
+                        group="defaults",
+                        prop="priority",
+                        default=Config.Defaults.priority,
+                        # ValueError raised if config string value not a member of JobPriority
+                        test=lambda value: Config.JobPriority(value),
+                    ),
+                    workers=self._get_property_or_default(
+                        settings=cfg,
+                        group="defaults",
+                        prop="workers",
+                        default=Config.Defaults.workers,
+                        test=lambda value: isinstance(value, int)
+                        and value >= 0,
+                        msg=f"must be an integer value >= 0.",
+                    ),
+                    cmdfiles=self._get_property_or_default(
+                        settings=cfg,
+                        group="defaults",
+                        prop="cmdfiles",
+                        default=Config.Defaults.cmdfiles,
+                        test=lambda value: isinstance(value, tuple)
+                        and all([isinstance(e, str) for e in value]),
+                        msg=f"must be a list or tuple of strings.",
+                    ),
                 )
-                self._validate_property(
-                    settings=cfg,
-                    group="defaults",
-                    prop="priority",
-                    # ValueError raised if config string value not a member of JobPriority
-                    test=lambda value: Config.JobPriority(value),
-                )
-                self._validate_property(
-                    settings=cfg,
-                    group="defaults",
-                    prop="workers",
-                    test=lambda value: isinstance(value, int) and value >= 0,
-                    msg=f"must be an integer value >= 0.",
-                )
-                self._validate_property(
-                    settings=cfg,
-                    group="defaults",
-                    prop="cmdfiles",
-                    test=lambda value: isinstance(value, tuple)
-                    and all([isinstance(e, str) for e in value]),
-                    msg=f"must be a list or tuple of strings.",
-                )
-                self.defaults = Config.Defaults(**cfg["defaults"])
 
             if "ssh" in cfg and all(
                 [
                     e in cfg["ssh"]
                     and self._validate_property(
-                        settings=cfg, group="ssh", prop=e
+                        settings=cfg, group="ssh", prop=e,
                     )
                     for e in ["server", "username", "password"]
                 ]
