@@ -1,17 +1,25 @@
-from typing import Any, Callable, Dict, List, Optional, Tuple
+from pathlib import Path
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    OrderedDict,
+    Tuple,
+)
 from types import ModuleType
 
+import collections
 import json
 import sys
 import tqdm
 import re
 import logging
-import datetime
-from pathlib import Path
 
 from solitude import TOOL_NAME, hookspecs
 from solitude.cache import Cache
-from solitude.commands import CommandBase
+from solitude.commands import CommandBase, update_jobinfo_for_multiple_commands
 import solitude.utils.ssh as ssh
 import solitude.plugins.default
 import solitude.plugins.csubmit
@@ -24,7 +32,7 @@ import pluggy
 from solitude.config import Config
 from solitude.slurmjob import SlurmJob
 
-logger = logging.getLogger(f"{TOOL_NAME}")
+logger = logging.getLogger(TOOL_NAME)
 
 
 COMMENT_MARKER = "#"
@@ -35,6 +43,8 @@ SOL_DOCKER_REPO_ALIASES = ["doduo1.umcn.nl", "doduo2.umcn.nl", "doduo.umcn.nl"]
 # this is the repository that will be used at core.run
 SOL_DOCKER_REPO = "doduo1.umcn.nl"
 
+CommandListType = OrderedDict[str, List[CommandBase]]
+
 
 def resolve_cmd_files(cmd_files: List[Path]) -> List[str]:
     commands: List[str] = []
@@ -64,8 +74,7 @@ def resolve_cmd_file(cmd_file: Path) -> List[str]:
 
 def shared_section(
     cmd_files: List[Path], cfg: Config
-) -> Tuple[List[CommandBase], Cache, Optional[Config.SSHConfig]]:
-    logger.debug(f"Starting action at: {datetime.datetime.now()}")
+) -> Tuple[CommandListType, Cache, Optional[Config.SSHConfig]]:
     # load plugins
     plugin_manager = setup_plugins(plugins=cfg.plugins)
     logger.debug(f"Read {len(plugin_manager.get_plugins())} command plugins")
@@ -75,13 +84,28 @@ def shared_section(
     cache.cleanup()
     # load command files
     cmds_strs = resolve_cmd_files(cmd_files=cmd_files)
-    cmds: List[CommandBase] = read_commands(
+    cmds: CommandListType = read_commands(
         commands=cmds_strs, cache=cache, plugin_manager=plugin_manager,
     )
     # retrieve ssh credentials
     return cmds, cache, cfg.ssh
 
 
+def get_unique_commands(commands: CommandListType) -> List[CommandBase]:
+    return [cmd_list[0] for cmd_list in commands.values()]
+
+
+class ChannelConnectFailedFilter(logging.Filter):
+    def filter(self, record):
+        return not (
+            record.name == "paramiko.transport"
+            and re.match(
+                r".*ecsh channel \d+ open FAILED: open failed: Connect failed.*",
+                record.getMessage(),
+            )
+        )
+
+
 class SpecialFormatter(logging.Formatter):
     def __init__(self, *args, **kwargs):
         super(SpecialFormatter, self).__init__(*args, **kwargs)
@@ -106,7 +130,9 @@ def log_setup(log_level: int = logging.DEBUG) -> logging.Logger:
     ch.setLevel(log_level)
     ch.setFormatter(fmt)
     log.addHandler(ch)
-    logging.getLogger("paramiko.transport").setLevel(logging.ERROR)
+    logger_pt = logging.getLogger("paramiko.transport")
+    logger_pt.setLevel(logging.ERROR)
+    logger_pt.addFilter(ChannelConnectFailedFilter())
     logging.getLogger("urllib3.connectionpool").setLevel(logging.ERROR)
     logging.getLogger("Cache").setLevel(logging.ERROR)
     return logging.getLogger(TOOL_NAME)
@@ -143,13 +169,21 @@ def match_plugin(pm: pluggy.PluginManager, cmd: str) -> ModuleType:
 
 def read_commands(
     commands: List[str], cache: Cache, plugin_manager: pluggy.PluginManager
-) -> List[CommandBase]:
-    return [
+) -> CommandListType:
+    cmds = [
         CommandBase(
             cmd=x, plugin=match_plugin(pm=plugin_manager, cmd=x), cache=cache
         )
         for x in commands
     ]
+    cmd_dict: CommandListType = collections.OrderedDict()
+    for cmd in cmds:
+        hashcode = cmd.hash
+        if hashcode in cmd_dict:
+            cmd_dict[hashcode].append(cmd)
+        else:
+            cmd_dict[hashcode] = [cmd]
+    return cmd_dict
 
 
 def execute_commands(
@@ -218,7 +252,8 @@ def link(
 ) -> bool:
     ssh_client = get_connected_ssh_client_from_config(cfg, verbose=False)
 
-    commands, cache, _ = shared_section(cmd_files=cmd_files, cfg=cfg)
+    commands_dict, cache, _ = shared_section(cmd_files=cmd_files, cfg=cfg)
+    commands = get_unique_commands(commands=commands_dict)
     cmd = commands[jobid - 1]
     # link jobid to slurm_jobid
     if SlurmJob.check_if_job_exists(ssh_client=ssh_client, jobid=slurm_jobid):
@@ -233,7 +268,8 @@ def get_log(
     cmd_files: List[Path], jobid: int, cfg: Config, active_polling: bool
 ) -> Optional[str]:
     ssh_client = get_connected_ssh_client_from_config(cfg, verbose=False)
-    commands, _, _ = shared_section(cmd_files=cmd_files, cfg=cfg)
+    commands_dict, _, _ = shared_section(cmd_files=cmd_files, cfg=cfg)
+    commands = get_unique_commands(commands=commands_dict)
     cmd = commands[jobid - 1]
     cmd.update_job_info(ssh_client=ssh_client)
     if cmd.job_info is not None:
@@ -256,7 +292,8 @@ def get_connected_ssh_client_from_config(
 
 
 def unlink(cmd_files: List[Path], jobid: int, cfg: Config) -> bool:
-    commands, cache, _ = shared_section(cmd_files=cmd_files, cfg=cfg)
+    commands_dict, cache, _ = shared_section(cmd_files=cmd_files, cfg=cfg)
+    commands = get_unique_commands(commands=commands_dict)
     cmd = commands[jobid - 1]
     if cmd.hash in cache:
         cache.delete(key=cmd.hash)
@@ -274,17 +311,18 @@ def _run_command_on_running_job(
 ):
     ssh_client = get_connected_ssh_client_from_config(cfg=cfg, verbose=False)
 
-    commands, _, credentials = shared_section(cmd_files=cmd_files, cfg=cfg)
+    commands_dict, _, credentials = shared_section(
+        cmd_files=cmd_files, cfg=cfg
+    )
+    commands = get_unique_commands(commands=commands_dict)
 
     # update selected commands only (use workers)
     subcmds = [cmd for idx, cmd in enumerate(commands) if idx + 1 in jobids]
 
-    def _update_cmd(cmd):
-        cmd.update_job_info(ssh_client=ssh_client)
-
-    execute_commands(
-        subcmds, map_fn=_update_cmd, workers=workers, show_progress_bar=True
+    update_jobinfo_for_multiple_commands(
+        commands=subcmds, ssh_client=ssh_client
     )
+
     ssh_client.verbose = True
 
     for idx, cmd in zip(jobids, subcmds):
@@ -294,7 +332,6 @@ def _run_command_on_running_job(
             ssh_client.exec_command(f"./{command} {cmd.job_info.id}")
         else:
             logger.info(f"Job {idx} is no longer running. Skipping...")
-    log_action_finished()
 
 
 def list_jobs(
@@ -304,10 +341,12 @@ def list_jobs(
     cfg: Config,
     selected_only: bool = False,
     show_progress_bar: bool = True,
+    show_duplicates: bool = False,
 ):
     ssh_client = get_connected_ssh_client_from_config(cfg=cfg, verbose=False)
 
-    commands, _, _ = shared_section(cmd_files=cmd_files, cfg=cfg)
+    commands_dict, _, _ = shared_section(cmd_files=cmd_files, cfg=cfg)
+    commands = get_unique_commands(commands=commands_dict)
 
     # update selected commands only (use workers)
     selected_ids = jobids
@@ -318,9 +357,19 @@ def list_jobs(
         for idx, cmd in enumerate(commands)
         if (idx + 1 in jobids) or not selected_only
     ]
+    mapped_cmds_list = [
+        cmd_list
+        for idx, cmd_list in enumerate(commands_dict.values())
+        if (idx + 1 in jobids) or not selected_only
+    ]
+
+    update_jobinfo_for_multiple_commands(
+        commands=subcmds, ssh_client=ssh_client
+    )
 
     def _update_cmd(cmd):
-        cmd.update(ssh_client=ssh_client)
+        cmd.update_state()
+        cmd.update_errors()
 
     execute_commands(
         subcmds,
@@ -330,13 +379,20 @@ def list_jobs(
     )
     logger.info("======Commands======")
     logger.info(CommandBase.header_str())
-    for idx, cmd in zip(jobids, subcmds):
+    for idx, cmd, mapped_cmds in zip(jobids, subcmds, mapped_cmds_list):
         logger.info(f'{idx:2d}{"*" if idx in selected_ids else " "} {cmd}')
-    log_action_finished()
-
-
-def log_action_finished():
-    logger.debug(f"Finished action at: {datetime.datetime.now()}")
+        if len(mapped_cmds) > 1:
+            logger.info(
+                f"    (There are {len(mapped_cmds)} lines that map to the above command"
+                + (
+                    f", run `job list` with the -d flag to list them here)"
+                    if not show_duplicates
+                    else ")"
+                )
+            )
+            if show_duplicates:
+                for lineidx, line in enumerate(mapped_cmds):
+                    logger.info(f"    {lineidx + 1:3d} {line.cmd}")
 
 
 def run(
@@ -352,13 +408,13 @@ def run(
 ):
     ssh_client = get_connected_ssh_client_from_config(cfg=cfg, verbose=False)
 
-    commands, cache, credentials = shared_section(cmd_files=cmd_files, cfg=cfg)
+    commands_dict, cache, credentials = shared_section(
+        cmd_files=cmd_files, cfg=cfg
+    )
+    commands = get_unique_commands(commands=commands_dict)
 
-    def _update_cmd(cmd):
-        cmd.update_job_info(ssh_client=ssh_client)
-
-    execute_commands(
-        commands, map_fn=_update_cmd, workers=workers, show_progress_bar=False
+    update_jobinfo_for_multiple_commands(
+        commands=commands, ssh_client=ssh_client
     )
 
     high_priority_job_available = not any(
@@ -381,12 +437,12 @@ def run(
     # update selected commands only (use workers)
     subcmds = [cmd for idx, cmd in enumerate(commands) if idx + 1 in jobids]
 
-    def _update_cmd2(cmd):
+    def _update_cmd(cmd):
         cmd.update_state()
         cmd.update_errors()
 
     execute_commands(
-        subcmds, map_fn=_update_cmd2, workers=workers, show_progress_bar=True
+        subcmds, map_fn=_update_cmd, workers=workers, show_progress_bar=True
     )
     ssh_client.verbose = True
 
@@ -451,8 +507,6 @@ def run(
             except Exception as e:
                 raise e
 
-    log_action_finished()
-
 
 def format_csubmit_str(
     cmdstr: str,
